/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "thermo16.h"

// ------------------------------------------------------------- PRIVATE MACROS 

// ADC resolutions

#define THERMO16_ADC_RESOLUTIONS_12bit 4096.0
#define THERMO16_ADC_RESOLUTIONS_10bit 1024.0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void thermo16_cfg_setup ( thermo16_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->an_pin = HAL_PIN_NC;

    cfg->resolution   = ANALOG_IN_RESOLUTION_12_BIT;
    cfg->vref         = 3.3;
}

THERMO16_RETVAL thermo16_init ( thermo16_t *ctx, thermo16_cfg_t *cfg )
{
    analog_in_config_t adc_cfg;

    analog_in_configure_default( &adc_cfg );
    adc_cfg.input_pin  = cfg->an_pin;

    if ( analog_in_open( &ctx->adc, &adc_cfg ) != ACQUIRE_FAIL )
    {
        return THERMO16_INIT_ERROR;
    }

    analog_in_set_vref_value( &ctx->adc, cfg->vref );
    analog_in_set_resolution( &ctx->adc, cfg->resolution );

    return THERMO16_OK;
}

thermo16_data_t thermo16_generic_read ( thermo16_t *ctx )
{
    thermo16_data_t rx_data;

    analog_in_read( &ctx->adc, &rx_data );
    
    return rx_data;
}

float thermo16_get_temperature ( thermo16_t *ctx, uint8_t temp_format )
{
    float voltage;
    float temperature;

    analog_in_read_voltage( &ctx->adc, &voltage );

    temperature = -(voltage * 100);

    if ( temp_format == THERMO16_TEMP_IN_KELVIN )
    {
        temperature = temperature + 273.15;
    }
    else if ( temp_format == THERMO16_TEMP_IN_FAHRENHEIT )
    {
        temperature = ( temperature * 9.0/5.0 ) + 32;
    }

    return temperature;
}

// ------------------------------------------------------------------------- END

